@@ -155,7 +155,7 @@ gdm_display_check_loop (GdmDisplay *disp)
       disp->last_start_time = now;
     }
 
-  disp->retry_count ++;
+  disp->retry_count++;
 
   return TRUE;
 }
@@ -368,7 +368,6 @@ gdm_display_manage (GdmDisplay *d)
  *
  * Stop services for a display
  */
-
 void
 gdm_display_unmanage (GdmDisplay *d)
 {
@@ -387,7 +386,7 @@ gdm_display_unmanage (GdmDisplay *d)
     whack_old_slave (d, TRUE /* kill_connection */);
     
     d->dispstat = DISPLAY_DEAD;
-    if (d->type != TYPE_STATIC)
+    if (d->type != TYPE_STATIC || d->removeconf)
 	gdm_display_dispose (d);
 
     gdm_debug ("gdm_display_unmanage: Display stopped");
@@ -409,12 +408,12 @@ count_session_limits (void)
 		GdmDisplay *d = li->data;
 		if (SERVER_IS_XDMCP (d)) {
 			if (d->dispstat == XDMCP_MANAGED)
-				xdmcp_sessions ++;
+				xdmcp_sessions++;
 			else if (d->dispstat == XDMCP_PENDING)
-				xdmcp_pending ++;
+				xdmcp_pending++;
 		}
 		if (SERVER_IS_FLEXI (d)) {
-			flexi_servers ++;
+			flexi_servers++;
 		}
 	}
 }
