@@ -43,7 +43,7 @@
 #include <pwd.h>
 #include <grp.h>
 
-#include <glib-2.0/glib.h>
+#include <glib.h>
 #include <glib/gi18n.h>
 
 #include "vicious.h"
@@ -217,6 +217,7 @@ static gchar *GdmGraphicalThemeColor;
 static gchar *GdmGraphicalThemeDir;
 static gchar *GdmGraphicalThemes;
 static gchar *GdmPreFetchProgram;
+static gchar *GdmUse24Clock;
 
 static gint GdmPositionX;
 static gint GdmPositionY;
@@ -227,6 +228,8 @@ static gint GdmBackgroundType;
 static gint GdmScanTime;
 static gint GdmMaxWait;
 static gint GdmFlexiReapDelayMinutes;
+static gint GdmBackgroundProgramInitialDelay = 30;
+static gint GdmBackgroundProgramRestartDelay = 30;
 
 static gboolean GdmAllowGtkThemeChange;
 static gboolean GdmTitleBar;
@@ -242,12 +245,12 @@ static gboolean GdmQuiver;
 static gboolean GdmShowGnomeFailsafe;
 static gboolean GdmShowXtermFailsafe;
 static gboolean GdmShowLastSession;
-static gchar * GdmUse24Clock;
 static gboolean GdmEntryCircles;
 static gboolean GdmEntryInvisible;
 static gboolean GdmGraphicalThemeRand;
 static gboolean GdmBroadcast;
 static gboolean GdmAllowAdd;
+static gboolean GdmRestartBackgroundProgram;
 
 /**
  * gdm_config_add_hash
@@ -413,6 +416,8 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_BROADCAST, &GdmBroadcast, &bool_type);
    gdm_config_add_hash (GDM_KEY_ALLOW_ADD, &GdmAllowAdd, &bool_type);
    gdm_config_add_hash (GDM_KEY_SOUND_ON_LOGIN, &GdmSoundOnLogin, &bool_type);
+   gdm_config_add_hash (GDM_KEY_RESTART_BACKGROUND_PROGRAM,
+      &GdmRestartBackgroundProgram, &bool_type);
 
    /* string values */
    gdm_config_add_hash (GDM_KEY_PATH, &GdmPath, &string_type);
@@ -523,6 +528,10 @@ gdm_config_init (void)
    gdm_config_add_hash (GDM_KEY_SCAN_TIME, &GdmScanTime, &int_type);
    gdm_config_add_hash (GDM_KEY_FLEXI_REAP_DELAY_MINUTES,
       &GdmFlexiReapDelayMinutes, &int_type);
+   gdm_config_add_hash (GDM_KEY_BACKGROUND_PROGRAM_INITIAL_DELAY,
+      &GdmBackgroundProgramInitialDelay, &int_type);
+   gdm_config_add_hash (GDM_KEY_BACKGROUND_PROGRAM_RESTART_DELAY,
+      &GdmBackgroundProgramRestartDelay, &int_type);
 }
 
 /**
@@ -531,7 +540,7 @@ gdm_config_init (void)
  * Get config file.  
  */
 static VeConfig *
-gdm_get_config (struct stat *statbuf)
+gdm_get_default_config (struct stat *statbuf)
 {
    int r;
 
@@ -544,13 +553,13 @@ gdm_get_config (struct stat *statbuf)
          return NULL;
       }
    } else {
-      VE_IGNORE_EINTR (r = g_stat (GDM_SYSCONFDIR_CONFIG_FILE, statbuf));
+      VE_IGNORE_EINTR (r = g_stat (GDM_DEFAULTS_CONF, statbuf));
       if (r < 0) {
          gdm_error (_("%s: No GDM configuration file: %s. Using defaults."),
-                      "gdm_config_parse", GDM_SYSCONFDIR_CONFIG_FILE);
+                      "gdm_config_parse", GDM_DEFAULTS_CONF);
          return NULL;
       } else {
-              config_file = GDM_SYSCONFDIR_CONFIG_FILE;
+              config_file = GDM_DEFAULTS_CONF;
       }
    }
 
@@ -566,13 +575,11 @@ gdm_get_config (struct stat *statbuf)
 static VeConfig *
 gdm_get_custom_config (struct stat *statbuf)
 {
-   VeConfig *retval;
-   gchar *file = g_strdup_printf ("%s-custom", GDM_SYSCONFDIR_CONFIG_FILE);
    int r;
 
-   VE_IGNORE_EINTR (r = g_stat (file, statbuf));
+   VE_IGNORE_EINTR (r = g_stat (GDM_CUSTOM_CONF, statbuf));
    if (r >= 0) {
-      custom_config_file = file;
+      custom_config_file = g_strdup (GDM_CUSTOM_CONF);
       return ve_config_new (custom_config_file);
    } else {
       return NULL;
@@ -678,7 +685,6 @@ gdm_config_to_string (gchar *key, gchar **retval)
    if (translated_hash != NULL) {
       gpointer val = gdm_config_hash_lookup (translated_hash, key);
       if (val) {
-         gchar *charval = (char *)val;
          *retval = g_strdup (val);
          return;
       }
@@ -714,7 +720,8 @@ gdm_config_to_string (gchar *key, gchar **retval)
 /**
  * gdm_compare_displays
  * 
- * Support function for loading displays from the gdm.conf file
+ * Support function for loading displays from the configuration
+ * file
  */
 int
 gdm_compare_displays (gconstpointer a, gconstpointer b)
@@ -1132,6 +1139,8 @@ static gboolean
 gdm_set_value (VeConfig *cfg, GdmConfigType *type, gchar *key, gboolean doing_update) 
 {
    gchar * realkey = gdm_config_hash_lookup (realkey_hash, key);
+   gchar *value;
+
    if (realkey == NULL) {
       return FALSE;
    }
@@ -1171,12 +1180,13 @@ gdm_set_value (VeConfig *cfg, GdmConfigType *type, gchar *key, gboolean doing_up
           while (list != NULL) {
              if (g_str_has_prefix ((char *)list->data, prefix) &&
                  g_str_has_suffix ((char *)list->data, "]")) {
+                gchar *transkey, *transvalue;
 
                 if (translated_hash == NULL)
                    translated_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
-                gchar    *transkey     = g_strdup_printf ("greeter/%s", (char *)list->data);
-                gchar    *transvalue   = ve_config_get_string (cfg, transkey);
+                transkey   = g_strdup_printf ("greeter/%s", (char *)list->data);
+                transvalue = ve_config_get_string (cfg, transkey);
 
                 g_hash_table_remove (translated_hash, transkey);
 
@@ -1195,7 +1205,7 @@ gdm_set_value (VeConfig *cfg, GdmConfigType *type, gchar *key, gboolean doing_up
        }
 
        /* Handle non-translated strings as normal */
-       gchar *value = ve_config_get_string (cfg, realkey);
+       value = ve_config_get_string (cfg, realkey);
        _gdm_set_value_string (key, value, doing_update);
        return TRUE;
    }
@@ -1272,7 +1282,6 @@ static void
 gdm_load_xservers (VeConfig *cfg)
 {
    GList *list, *li;
-   GSList *xli;
 
    /* Find server definitions */
    list = ve_config_get_sections (cfg);
@@ -1539,30 +1548,36 @@ check_servauthdir (struct stat *statbuf)
     /* Enter paranoia mode */
     VE_IGNORE_EINTR (r = g_stat (GdmServAuthDir, statbuf));
     if G_UNLIKELY (r < 0) {
-            gchar *s = g_strdup_printf
-                    (C_(N_("Server Authorization directory "
-                           "(daemon/ServAuthDir) is set to %s "
-                           "but this does not exist. Please "
-                           "correct GDM configuration and "
-                           "restart GDM.")), GdmServAuthDir);
-        if (GdmConsoleNotify)
-                    gdm_text_message_dialog (s);
+            if (GdmConsoleNotify) {
+               gchar *s = g_strdup_printf
+                       (C_(N_("Server Authorization directory "
+                              "(daemon/ServAuthDir) is set to %s "
+                              "but this does not exist. Please "
+                              "correct GDM configuration and "
+                              "restart GDM.")), GdmServAuthDir);
+
+               gdm_text_message_dialog (s);
+               g_free (s);
+            }
+
             GdmPidFile = NULL;
-            g_free (s);
             gdm_fail (_("%s: Authdir %s does not exist. Aborting."), "gdm_config_parse", GdmServAuthDir);
     }
 
     if G_UNLIKELY (! S_ISDIR (statbuf->st_mode)) {
-            gchar *s = g_strdup_printf
-                    (C_(N_("Server Authorization directory "
-                           "(daemon/ServAuthDir) is set to %s "
-                           "but this is not a directory. Please "
-                           "correct GDM configuration and "
-                           "restart GDM.")), GdmServAuthDir);
-        if (GdmConsoleNotify)
-                    gdm_text_message_dialog (s);
+            if (GdmConsoleNotify) {
+               gchar *s = g_strdup_printf
+                       (C_(N_("Server Authorization directory "
+                              "(daemon/ServAuthDir) is set to %s "
+                              "but this is not a directory. Please "
+                              "correct GDM configuration and "
+                              "restart GDM.")), GdmServAuthDir);
+
+                gdm_text_message_dialog (s);
+                g_free (s);
+            }
+
             GdmPidFile = NULL;
-            g_free (s);
             gdm_fail (_("%s: Authdir %s is not a directory. Aborting."), "gdm_config_parse", GdmServAuthDir);
     }
 }
@@ -1652,7 +1667,6 @@ gdm_load_config_option (gpointer key_in, gpointer value_in, gpointer data)
    gchar *key               = (gchar *)key_in;
    GdmConfigType *type      = (GdmConfigType *)value_in;
    GdmConfigFiles *cfgfiles = (GdmConfigFiles *)data;
-   gboolean retval;
    gboolean custom_retval;
 
    if (type != NULL) {
@@ -1714,7 +1728,7 @@ gdm_config_parse (void)
     * main configuration file.  If cfg is missing, then GDM will 
     * use the built-in defaults found in gdm.h.
     */
-   cfg                      = gdm_get_config (&statbuf);
+   cfg                      = gdm_get_default_config (&statbuf);
    config_file_mtime        = statbuf.st_mtime;
    custom_cfg               = gdm_get_custom_config (&statbuf);
    custom_config_file_mtime = statbuf.st_mtime;
@@ -1789,14 +1803,17 @@ gdm_config_parse (void)
          GdmAutomaticLogin = NULL;
          GdmTimedLogin     = NULL;
       } else {
-         gchar *s = g_strdup_printf (C_(N_("XDMCP is disabled and GDM "
-                                   "cannot find any static server "
-                                   "to start.  Aborting!  Please "
-                                   "correct the configuration "
-                                   "and restart GDM.")));
-         gdm_text_message_dialog (s);
+         if (GdmConsoleNotify) {
+            gchar *s = g_strdup_printf (C_(N_("XDMCP is disabled and GDM "
+                                      "cannot find any static server "
+                                      "to start.  Aborting!  Please "
+                                      "correct the configuration "
+                                      "and restart GDM.")));
+            gdm_text_message_dialog (s);
+            g_free (s);
+         }
+
          GdmPidFile = NULL;
-         g_free (s);
          gdm_fail (_("%s: XDMCP disabled and no static servers defined. Aborting!"), "gdm_config_parse");
       }
    }
@@ -1814,57 +1831,66 @@ gdm_config_parse (void)
 
    /* Set GdmUserId and GdmGroupId */
    if G_UNLIKELY (pwent == NULL) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM user '%s' does not exist. "
-                                        "Please correct GDM configuration "
-                                        "and restart GDM.")), GdmUser);
-      if (GdmConsoleNotify)
+
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM user '%s' does not exist. "
+                                           "Please correct GDM configuration "
+                                           "and restart GDM.")), GdmUser);
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
 
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: Can't find the GDM user '%s'. Aborting!"), "gdm_config_parse", GdmUser);
    } else {
       GdmUserId = pwent->pw_uid;
    }
 
    if G_UNLIKELY (GdmUserId == 0) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM user is set to be root, but "
-                                        "this is not allowed since it can "
-                                        "pose a security risk.  Please "
-                                        "correct GDM configuration and "
-                                        "restart GDM.")));
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM user is set to be root, but "
+                                           "this is not allowed since it can "
+                                           "pose a security risk.  Please "
+                                           "correct GDM configuration and "
+                                           "restart GDM.")));
+
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: The GDM user should not be root. Aborting!"), "gdm_config_parse");
    }
 
    grent = getgrnam (GdmGroup);
 
    if G_UNLIKELY (grent == NULL) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM group '%s' does not exist. "
-                                        "Please correct GDM configuration "
-                                        "and restart GDM.")), GdmGroup);
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM group '%s' does not exist. "
+                                           "Please correct GDM configuration "
+                                           "and restart GDM.")), GdmGroup);
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: Can't find the GDM group '%s'. Aborting!"), "gdm_config_parse", GdmGroup);
    } else  {
       GdmGroupId = grent->gr_gid;   
    }
 
    if G_UNLIKELY (GdmGroupId == 0) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM group is set to be root, but "
-                                        "this is not allowed since it can "
-                                        "pose a security risk. Please "
-                                        "correct GDM configuration and "
-                                        "restart GDM.")));
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM group is set to be root, but "
+                                           "this is not allowed since it can "
+                                           "pose a security risk. Please "
+                                           "correct GDM configuration and "
+                                           "restart GDM.")));
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: The GDM group should not be root. Aborting!"), "gdm_config_parse");
    }
 
@@ -1895,9 +1921,10 @@ gdm_config_parse (void)
 
    /* Check the serv auth and log dirs */
    if G_UNLIKELY (ve_string_empty (GdmServAuthDir)) {
-       if (GdmConsoleNotify)
+       if (GdmConsoleNotify) {
           gdm_text_message_dialog
              (C_(N_("No daemon/ServAuthDir specified in the GDM configuration file")));
+       }
        GdmPidFile = NULL;
        gdm_fail (_("%s: No daemon/ServAuthDir specified."), "gdm_config_parse");
    }
@@ -1922,33 +1949,38 @@ gdm_config_parse (void)
    check_servauthdir (&statbuf);
 
    if G_UNLIKELY (statbuf.st_uid != 0 || statbuf.st_gid != GdmGroupId)  {
-      gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
-                                        "(daemon/ServAuthDir) is set to %s "
-                                        "but is not owned by user %s and group "
-                                        "%s. Please correct the ownership or "
-                                        "GDM configuration and restart "
-                                        "GDM.")), GdmServAuthDir, GdmUser, GdmGroup);
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
+                                           "(daemon/ServAuthDir) is set to %s "
+                                           "but is not owned by user %s and group "
+                                           "%s. Please correct the ownership or "
+                                           "GDM configuration and restart "
+                                           "GDM.")), GdmServAuthDir,
+		   			gdm_root_user (), GdmGroup);
          gdm_text_message_dialog (s);
-         GdmPidFile = NULL;
          g_free (s);
-         gdm_fail (_("%s: Authdir %s is not owned by user %s, group %s. Aborting."),
+      }
+
+      GdmPidFile = NULL;
+      gdm_fail (_("%s: Authdir %s is not owned by user %s, group %s. Aborting."),
                      "gdm_config_parse", GdmServAuthDir, gdm_root_user (), GdmGroup);
    }
 
    if G_UNLIKELY (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRWXG|S_ISVTX))  {
-      gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
-                                 "(daemon/ServAuthDir) is set to %s "
-                                 "but has the wrong permissions: it "
-                                 "should have permissions of %o. "
-                                 "Please correct the permissions or "
-                                 "the GDM configuration and "
-                                 "restart GDM.")), GdmServAuthDir,
-                                 (S_IRWXU|S_IRWXG|S_ISVTX));
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
+                                    "(daemon/ServAuthDir) is set to %s "
+                                    "but has the wrong permissions: it "
+                                    "should have permissions of %o. "
+                                    "Please correct the permissions or "
+                                    "the GDM configuration and "
+                                    "restart GDM.")), GdmServAuthDir,
+                                    (S_IRWXU|S_IRWXG|S_ISVTX));
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: Authdir %s has wrong permissions %o. Should be %o. Aborting."), "gdm_config_parse", 
                   GdmServAuthDir, statbuf.st_mode, (S_IRWXU|S_IRWXG|S_ISVTX));
    }
