@@ -18,12 +18,14 @@
 
 #include "config.h"
 
-#include <libgnome/libgnome.h>
+#include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <string.h>
 
+#include "gdm.h"
 #include "gdmwm.h"
 #include "gdmcommon.h"
+#include "gdmconfig.h"
 #include "gdmlanguages.h"
 #include "greeter.h"
 #include "greeter_configuration.h"
@@ -52,7 +54,7 @@ greeter_langauge_initialize_model (void)
   GList *list, *li;
   GtkTreeIter iter;
 
-  list = gdm_lang_read_locale_file (GdmLocaleFile);
+  list = gdm_lang_read_locale_file (gdm_config_get_string (GDM_KEY_LOCALE_FILE));
 
   lang_model = gtk_list_store_new (NUM_COLUMNS,
 				   G_TYPE_STRING,
@@ -160,7 +162,8 @@ greeter_language_get_language (const char *old_language)
 	  g_free (current_name);
 	  g_free (saved_name);
 
-	  savelang = gdm_common_query (primary_message, secondary_message, _("Make _Default"), _("Just For _This Session"), TRUE);
+	  savelang = gdm_wm_query_dialog (primary_message, secondary_message,
+		_("Make _Default"), _("Just For _This Session"), TRUE);
 	  g_free (primary_message);
 	  g_free (secondary_message);
 	}
@@ -203,9 +206,12 @@ tree_row_activated (GtkTreeView         *view,
   }
 }
 
+/*
+ * The button with this handler appears in the F10 menu, so it
+ * cannot depend on callback data being passed in.
+ */
 void
-greeter_action_language (GreeterItemInfo *info,
-			 gpointer         user_data)
+greeter_language_handler (void)
 {
   GtkWidget *view = NULL;
 
@@ -290,7 +296,7 @@ greeter_action_language (GreeterItemInfo *info,
 			"changed",
 			(GCallback) selection_changed,
 			NULL);
-      g_signal_connect(G_OBJECT (view),
+      g_signal_connect (G_OBJECT (view),
                         "row_activated",
                         (GCallback) tree_row_activated,
                         NULL);
@@ -308,7 +314,7 @@ greeter_action_language (GreeterItemInfo *info,
 			       GTK_TREE_MODEL (lang_model));
       selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
       if (selection != NULL)
-	gtk_tree_selection_select_path (selection, gtk_tree_path_new_first());
+	gtk_tree_selection_select_path (selection, gtk_tree_path_new_first ());
     }
   switch (gtk_dialog_run (GTK_DIALOG (dialog)))
     {
