@@ -17,8 +17,8 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
-#include <config.h>
-#include <glib.h>
+#include "config.h"
+
 #include <string.h>
 #include <unistd.h>
 #include <stdlib.h>
@@ -27,9 +27,10 @@
 #include <sys/stat.h>
 #include <time.h>
 
+#include <glib.h>
+
 #include "ve-i18n.h"
 #include "ve-misc.h"
-
 #include "ve-config.h"
 
 typedef struct _VeSection VeSection;
@@ -242,7 +243,7 @@ add_key_line (VeConfig *config, VeSection *section,
 						 section->name,
 						 key);
 		line->key = strchr (line->fullkey, '/');
-		line->key ++;
+		line->key++;
 	}
 	line->string = g_strdup (val);
 
@@ -324,7 +325,7 @@ read_config (VeConfig *config)
 		} else if ((eq = strchr (nows, '=')) != NULL) {
 			char *val;
 			*eq = '\0';
-			eq ++;
+			eq++;
 			val = g_strcompress (eq);
 			add_key_line (config, section,
 				      nows /* key */,
@@ -337,7 +338,7 @@ read_config (VeConfig *config)
 		/* Note this is a hard limit to avoid run away files.
 		   Do we really expect files longer then 5000 lines?
 		   This is to be ultra anal for security's sake. */
-		cnt ++;
+		cnt++;
 		if (cnt > 5000)
 			break;
 	}
